package org.snap.dresources;

import dermi.exception.*;
import org.planet.p2pcm.*;
import org.planet.snap.*;

import java.io.*;
import java.net.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

public class Download extends HttpServlet {	
	private static final long serialVersionUID = 235645734;
	
	private ServletContext ctx;
	private Application jounin_app;
	private String archive;
	private String separator;
	
	public void init (ServletConfig config) throws ServletException {
		super.init (config);
		ctx = config.getServletContext();
		jounin_app = (Application) ctx.getAttribute ("snapApp");
		
		separator = System.getProperty("file.separator");
		archive = "."+separator+"shared-data"+separator;
	}
	
	protected void processRequest(HttpServletRequest request, HttpServletResponse response) throws ServletException, java.io.IOException, RemoteException, ComponentNotRegisteredException, ComponentInitializationException, InterfaceNotFoundException {
		HttpSession session = request.getSession(true);
		
		//Get the main manager application
		Manager manager = (Manager) session.getAttribute("manager"); 
		if(manager==null) {
			manager = new Manager(jounin_app);
			session.setAttribute("manager", manager);
		}
		
		//Get the request path (starting after "download")
		String reqPath = request.getPathInfo();
		
		//Get the relative path and the name inside the resources
		String[] parts =  reqPath.split("/");
		String name = "";
		Vector<String> path = new Vector<String>();
		if(parts.length==1){
			name = parts[0];
		}else if(parts.length>1){
			name = parts[parts.length-1];
			for(int i=1; i<parts.length-1; i++)
				path.add(parts[i]);	
		}else{
			throw new ServletException("Download Path Invalid !");
		}		
		
		ResourcesElem elem;
		try{
			elem = manager.getResource(path, name);
		}catch(ManagerException e){
			throw new ServletException("Manager Exception");
		}
		
		//Read the input stream from URL
		URL urla = jounin_app.getStorageContext().retrieveURL(elem.getId());
		InputStream in = urla.openStream(); 
			
		//Read the file and prints to the browser
		OutputStream out = response.getOutputStream();
		flushFile(in, out);
	}
	
	public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		try {
			processRequest(request, response);
		} catch (ServletException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (ComponentNotRegisteredException e) {
			e.printStackTrace();
		} catch (ComponentInitializationException e) {
			e.printStackTrace();
		} catch (InterfaceNotFoundException e) {
			e.printStackTrace();
		}
	}
	
	public void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		try {
			processRequest(request, response);
		} catch (ServletException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (ComponentNotRegisteredException e) {
			e.printStackTrace();
		} catch (ComponentInitializationException e) {
			e.printStackTrace();
		} catch (InterfaceNotFoundException e) {
			e.printStackTrace();
		}
	}
	
	private void flushFile(InputStream in, OutputStream out) throws ServletException {
		// Create the byte array to hold the data
        byte[] bytes = new byte[100];
    
        try{
	        // Read in the bytes and send
	        while (in.read(bytes) >= 0) {
	            out.write(bytes);
	        }
        }catch(IOException e){
        	throw new ServletException("Error while reading the stream");
        }
        
        // Close the input stream
        try {
        	in.close();
        	out.close();
        }catch(IOException e){
        	throw new ServletException("Error while closing the streams");
        }
	}
} 
