package org.snap.dresources;

import java.io.*;
import java.net.*;
import java.nio.MappedByteBuffer;
import java.nio.channels.FileChannel;
import java.util.*;


import dermi.*;
import dermi.bittorrent.*;
import dermi.core.*;
import dermi.util.pastry.*;

import org.planet.snap.*;
import org.planet.p2pcm.*;

import com.oreilly.servlet.multipart.*;

import dermi.exception.RemoteException;

public class Manager {
	//Component ID
	private String component_name;
	// Main p2p component
	private static Resources component = null;
	
	private DHTHandler dht;
	private String context;
	private String temp;
	private String archive;
	private String separator;
	private String server;
	
	public Manager(Application app) throws RemoteException{
		dht = app.getStorageContext();
		server = "http://"+app.getServerName()+":"+app.getServerPort();
		context = app.getContext();
		
		component_name = "components/resources";
		
		separator= System.getProperty("file.separator");
		temp = System.getProperty("java.io.tmpdir")+separator;
		//TODO: Get the path to the context dinamically !!
		archive = "./work/Jetty__33333__"+context+"/webapp/torrents/";
	}
	
	public Collection getResources(Vector<String> path) throws RemoteException, ManagerException {
		getComponent();
		// A ComponentModel bug don't let to pass 'null' as argument
		if(path==null) path =new Vector<String>();
		return component.getResources(path);
	}
	
	public ResourcesElem getResource(Vector<String> path, String name) throws RemoteException, ManagerException {
		getComponent();
		// A ComponentModel bug don't let to pass 'null' as argument
		if(path==null) path =new Vector<String>();
		return component.getResource(path, name);
	}
	
	public ResourcesElem.ElemType getResourceType(Vector<String> path, String name) throws RemoteException, ManagerException {
		getComponent();
		// A ComponentModel bug don't let to pass 'null' as argument
		if(path==null) path =new Vector<String>();
		return component.getResourceType(path, name);
	}
	
	public void addFile(Vector<String> path, FilePart file) throws IOException, RemoteException, InterruptedException, ManagerException {
		getComponent();
		String fileName = file.getFileName();
		File f = null;
		
		// A ComponentModel bug don't let to pass 'null' as argument
		if(path==null) path =new Vector<String>();
		
		// Make the code of this file
		Vector<String> file_path = new Vector<String>(path);
		file_path.add(fileName);
		Id id = getFileCode(file_path);
		String key = ((rice.pastry.NodeId)id).toStringFull();
		
		// Save the file to local temp folder
		if (fileName != null) {                           
			f = new File(temp + key);
			file.writeTo(f);
		} else {
			throw new ManagerException("Can not save the file.");
		}
		long size = f.length();
		
		if(size>2000000){
			//Copy the file to the torrents folder
			File bigfile = new File(archive+key);
			bigfile.createNewFile();
			copy(f, bigfile);
			
			//Get the file URL
			String urlString = server+"/"+context+"/torrents/"+key;
			String urlTorrent = server+"/"+context+"/torrents/"+key+".torrent";
			URL url = new URL(urlTorrent);
			
			//Creates torrent
			BittorrentHandler ah = SingletonHandler.getInstance("dermi.bittorrent.azureus.AzureusHandler");
			ah.makeTorrent(urlString, archive+key);
			if(!ah.isConnected())
				ah.connect();
			ah.addFind(archive+key+".torrent", archive);
			
			//System.out.println("!!! Hosting file "+hash);
			//ah.host(hash);
	        //byte[] bytes = readTorrent(archive+key+".torrent");
			dht.insertURL(id, url);
			System.out.println("ADDING TO PATH: "+path);
			component.addFile(path, fileName+".torrent", id, ((rice.pastry.NodeId)id).toStringFull()+".torrent");
		}else {
			// Getting URL
			String urlString = f.toURL().toString();
			URL url = new URL(urlString);
			
			// Save the file to DHT	(a copy is added to storage folder)	
			dht.insertURL(id, url);
			component.addFile(path, fileName, id, ((rice.pastry.NodeId)id).toStringFull());
		}
		
		// Deletes the temporally file
		f.delete();
	}
	
	public void addFolder(Vector<String> path, String name) throws RemoteException, ManagerException {
		getComponent();
		// A ComponentModel bug don't let to pass 'null' as argument
		if(path==null) path =new Vector<String>();
		component.addFolder(path, name);
	}
	
	public void delete(Vector<String> path, String name) throws RemoteException, ManagerException {
		getComponent();
		// A ComponentModel bug don't let to pass 'null' as argument
		if(path==null) path =new Vector<String>();
		component.delete(path,name, dht);
	}
	
	private Id getFileCode(Vector path){
		String code = "";
		Iterator it = path.iterator();
		for (int i=0; it.hasNext(); i++) {										
			code += "-" + (String)it.next();
		}
		
		return HashGenerator.generateHash(code);
	}
	
	private void getComponent() throws ManagerException {
		//Try if the component is active
		try {
			System.out.println("Calling test method");
			component.test(); // Only to test if the component exists and its active
			System.out.println("Test method succesfull");
		} catch(Exception e) {
			System.out.println("Test method fails");
			System.out.println("Creating Resources component");
			
			ComponentFactory fac;
			try{
				fac = (ResourcesFactory) ComponentUtil.getComponentFactory ("p2p://resources-component.urv.net");
			}catch(Exception ee){
				throw new ManagerException("Impossible to get the component factory !");
			}
			
			System.out.println("I've the factory !");
			
			try{
				System.out.println("!!!!!!-----> ");
				component = (Resources) fac.createInstance ("org.snap.Resources", component_name);
			}catch(Exception ee){
				throw new ManagerException("Impossible to create the component !");
			}	
			
			System.out.println("We get the component");
		}
	}
	
	private void copy(File source, File dest) throws IOException {
	     FileChannel in = null, out = null;
	     try {          
	          in = new FileInputStream(source).getChannel();
	          out = new FileOutputStream(dest).getChannel();

	          long size = in.size();
	          MappedByteBuffer buf = in.map(FileChannel.MapMode.READ_ONLY, 0, size);

	          out.write(buf);

	     } finally {
	          if (in != null)          in.close();
	          if (out != null)     out.close();
	     }
	}
}
