package org.snap.dresources;

import org.planet.p2pcm.*;

import dermi.*;
import dermi.core.*;
import dermi.exception.*;
import dermi.annotation.*;

import java.util.*;

@DCMImplementation
public class ResourcesImpl extends Component implements Resources, StatefulReplica {
	private static final long serialVersionUID = 56453;
	
	private ResourcesElem _root;
	
	// Dermi constructor: REQUIRED
	public ResourcesImpl() {
	}
	
	// Dermi constructor: REQUIRED
	public ResourcesImpl (Properties env) throws RemoteException {
		super (env);
	
		_root = new ResourcesElem(ResourcesElem.ElemType.FOLDER, null, null);
		
		// Load replica state: as in any other stateful Dermi remote object
	    super.loadReplicaState();
	}
	
	public void addFolder(Vector<String> path, String name) throws RemoteException {
		ResourcesElem wr = getPathResources(path);
		
		try{
			wr.addResource(name, ResourcesElem.ElemType.FOLDER, null, null);
		}catch(ResourcesException e){
			throw new RemoteException("Impossible to add the folder "+name);
		}
	}
	
	public void addFile(Vector<String> path, String name, Id hash, String storage) throws RemoteException {
		ResourcesElem wr = getPathResources(path);
		
		try{
			wr.addResource(name, ResourcesElem.ElemType.FILE, hash, storage);
		}catch(ResourcesException e){
			throw new RemoteException("Impossible to add the folder "+name);
		}	
	}
	
	public void delete(Vector<String> path, String name, DHTHandler dht) throws RemoteException {
		ResourcesElem wr = getPathResources(path);
		
		ResourcesElem old = (ResourcesElem)wr.getElems().get(name);
		wr.getElems().remove(name);
		if(old.getType()==ResourcesElem.ElemType.FILE) {
			dht.removeURL(old.getId());
		}else if(old.getType()==ResourcesElem.ElemType.FOLDER) {
			delTree(old, dht);
		}
	}
	
	private void delTree(ResourcesElem tree, DHTHandler dht) throws RemoteException{		
		Iterator it = tree.getElems().keySet().iterator();
		while(it.hasNext()) {
			String n = (String)it.next();
			ResourcesElem elem = (ResourcesElem)tree.getElems().get(n);
			if(elem.getType()==ResourcesElem.ElemType.FILE) {
				dht.removeURL(elem.getId());
			}else if(elem.getType()==ResourcesElem.ElemType.FOLDER) {
				delTree(elem, dht);
			}
		}
	}
	
	public Collection getResources(Vector<String> path) throws RemoteException {
		ResourcesElem wr = getPathResources(path);
		return wr.getElems().keySet();
	}
	
	public ResourcesElem getResource(Vector<String> path, String name) throws RemoteException {
		ResourcesElem wr = getPathResources(path);
		return (ResourcesElem)wr.getElems().get(name);
	}
	
	public ResourcesElem.ElemType getResourceType(Vector<String> path, String name) throws RemoteException {
		ResourcesElem wr = getPathResources(path);
		return ((ResourcesElem)wr.getElems().get(name)).getType();
	}
	
	private ResourcesElem getPathResources(Vector<String> path) throws RemoteException {
		ResourcesElem actual = _root;
		
		if(path==null) return _root;
		
		Iterator it = path.iterator();
		for (int i=0; it.hasNext(); i++) {										
			String id = (String) it.next();
			if(actual.getElems().containsKey(id)){
				ResourcesElem tmp = (ResourcesElem)actual.getElems().get(id);
				ResourcesElem.ElemType type = tmp.getType();
				if(type==ResourcesElem.ElemType.FILE) {
					throw new RemoteException("The path contains a file name");
				}else if(type==ResourcesElem.ElemType.FOLDER) {
					actual = tmp;
				}
			}else{
				throw new RemoteException("Invalid path");
			}
		}
		
		return actual;
	}
	
	/*
	 * Useful for test if the component is active
	 */
	public boolean test() {
		return true;
	}
	
	/**
	 * The queryInterface method must return a suitable component implementation
	 * for the specified interface
	 * @param interf String The desired interface implementation
	 * @return ComponentInterface The desired implementation instance
	 * @throws InterfaceNotFoundException If the component does not provide an implementation for such specified interface
	 */
	public ComponentInterface queryInterface (String interf) throws InterfaceNotFoundException {
		try {
			if (interf.equals ("org.planet.p2pcm.ComponentInterface") || interf.equals ("org.snap.Resources")) {
				return (ComponentInterface) this.copy();
			} else {
				throw new InterfaceNotFoundException ("Interface " + interf + " not implemented.");
			}
		} catch (RemoteException re) {
			throw new InterfaceNotFoundException ("Unable to query for interface " + interf + ": " + re.getMessage());
		}
	}

	/**
	 * Method for loading state into this object
	 */
	public void loadRemoteObjectState() throws RemoteException {
		ArrayList temp = (ArrayList) super.loadState();
		_root = (ResourcesElem) temp.get(0);
		System.out.println ("[Resources] - state loaded from network.");    
	}

	/**
	 * Method for obtaining this object's state
	 */
	public java.io.Serializable getRemoteObjectState() {
		ArrayList temp = new ArrayList();
		temp.add (0, _root);
		return (java.io.Serializable) temp;
	}
}
